########################################################################
#                                                                      #
#  Zestaw skryptw KaduPro - wtyczek rozszerzajcych moliwoci        #
#  komunikatora Kadu. Do napisania skryptw wykorzystalimy jzyk      #
#  skryptowy TCL.                                                      #
#  KaduPro dostpne jest na licencji GPL (to znaczy moecie sobie      #
#  robi co chcecie, ale to co zrobicie te musicie opublikowa na     #
#  GPL'u)                                                              #
#  Wszystkie skargi, poprawki, zaalenia prosimy umieszcza na         #
#  oficjalnym forum komunikatora kadu: www.kadu.net/forum              #
#                                                                      #
#  Autorzy projektu:                                                   #
#  Googie - gowny developer, koordynator (autor moduu tcl_scripting  #
#  Ciuciu, Sad Mephisto - developerzy                                  #
#                                                                      #
#  Testerzy wersji niestabilnych (tj tych co formatuj dyski i robi   #
#  inn kaszan:                                                       #
#  Sajmon, Blaze, Attis                                                #
#                                                                      #
#  Najwiesz wersj KaduPro mona znale na oficjalnej stronie      #
#  projektu: http://scripts.one.pl/tcl4kadu/                           #
#                                                                      #
########################################################################  

### For developping - uncomment to enable debugger:
#source $module(scriptpath)/debugger.tcl


########################################################################
###   starting KaduPro - setting basic constatns                     ###
########################################################################

### Setting KaduPro and required tcl_scripting module versions:
set KaduProVersion $module(version)-$module(codename)

set req1 0.6
set req2 0

set ver1 [join [lrange [split $module(version) .] 0 1] .]
set ver2 [lindex [split $module(version) .] end]


### Display an error message if tcl_scripting is too old
if {$ver1 < $req1} {
    msgbox msg "Wersja moduu TCL jest za stara ($module(version)). Wymagana jest $req1.$req2."
} elseif {$ver1 == $req1 && $ver2 < $req2} {
    msgbox msg "Wersja moduu TCL jest za stara ($module(version)). Wymagana jest $req1.$req2."
}


### Set Iso8859-2 (central europe) encoding, and display possible error message
if {[lsearch [encoding names] iso8859-2] > -1} {
    encoding system iso8859-2
} else {
    msgbox msg "Twoja instalacja biblioteki TCL (nie modulu!) nie pozwala\nna ustawienie kodowania znakow w standardzie ISO8859-2.\nMoze to powodowac niepoprawne wyswietlanie polskich znakow w KaduPro!"
}


### Allow to use '//' types of comments
proc // {args} {
	return
}


### Reading qtrc color sheme for QT (and use it in tcl)
if {[file readable $env(HOME)/.qt/qtrc]} {
    set fd [open $env(HOME)/.qt/qtrc r]
    set data [read $fd]
    close $fd
    set sw1 0
    set sw2 0

    foreach line [split $data "\n"] {
        if {$sw1} {
            set line [string map {^e ^} $line]
            set sp [split $line =]
            set type [lindex $sp 0]
            if {$sw2 > 3} {
                break
            } elseif {"active" == $type} {
                set sp2 [split [lindex $sp 1] ^]
                set QT_look(fg) [lindex $sp2 6]            ;# Foreground
                set QT_look(bg) [lindex $sp2 9]            ;# Background
                set QT_look(bt_fg) [lindex $sp2 8]         ;# Buttons foreground
                set QT_look(bt_bg) [lindex $sp2 1]         ;# Buttons background
                set QT_look(win_fg) [lindex $sp2 0]        ;# Windows foreground
                set QT_look(win_bg) [lindex $sp2 10]       ;# Windows background
                set QT_look(win_sel_fg) [lindex $sp2 13]   ;# Selected windows foreground
                set QT_look(win_sel_bg) [lindex $sp2 12]   ;# Selected windows background
                set QT_look(link) [lindex $sp2 14]         ;# Links
                set QT_look(visited) [lindex $sp2 15]      ;# Visited links
                set QT_look(bt_act_fg) "#"                 ;# Active buttons foreground
                set QT_look(bt_act_bg) "#"                 ;# Active buttons background

                for {set i 1} {$i < 7} {incr i} {
                    set c [scan [string index $QT_look(bt_bg) $i] "%x"]
                    if {$c < 15} {
                        incr c
                    }

                    append QT_look(bt_act_bg) [format "%x" $c]
    
                    set c [scan [string index $QT_look(bt_fg) $i] "%x"]
                    if {$c < 15} {
                        incr c
                    }

                    append QT_look(bt_act_fg) [format "%x" $c]
                }
    
                incr sw2
            }
        } else {
            if {"$line" == "\[Palette\]"} {
                set sw1 1
            }
        }
    }
}


### Moving old KP configuration to new place
if {![file exists $kadu(configdir)/kadupro]} {
    file mkdir $kadu(configdir)/kadupro
}

if {[file exists $kadu(configdir)/kadupro.conf]} {
    file rename $kadu(configdir)/kadupro.conf $kadu(configdir)/kadupro/kadupro.conf
}

if {[file exists $kadu(configdir)/addresses.kp]} {
    file rename $kadu(configdir)/addresses.kp $kadu(configdir)/kadupro/addresses.kp
}


### Kadu configuration directory and data directory
set config_dir "$kadu(configdir)/kadupro"

if {[file exists $kadu(datadir)/kadu/scripts/kadupro]} {
    set data_dir "$kadu(datadir)/kadu/scripts"
} else {
    set data_dir "$config_dir"
}

set kpicons "$module(scriptpath)/kadupro/icons"



########################################################################
###   loading external modules and libraries                         ###
########################################################################

### GUI:
package require Tk
wm withdraw .

### Libraries
source $module(scriptpath)/kadupro/libs/anigif.tcl

### Default configuration
set RememberAdresses 10

### Loading KP components !
### if you don't want some components to load.. put "#" before them :P

### library-modules. They realy do nothing, but should be at begining:
	source $module(scriptpath)/kadupro/scripts/translator.kkp
	source $module(scriptpath)/kadupro/scripts/translator_dict.kkp
	source $module(scriptpath)/kadupro/scripts/locale.kkp
	source $module(scriptpath)/kadupro/scripts/library.kkp
	source $module(scriptpath)/kadupro/scripts/status_table.kkp

### configuration comes first, and all other stuff comes next
	source $module(scriptpath)/kadupro/scripts/config.kkp

	source $module(scriptpath)/kadupro/scripts/shortcuts.kkp
	source $module(scriptpath)/kadupro/scripts/notifier.kkp
	source $module(scriptpath)/kadupro/scripts/antistrings.kkp
	source $module(scriptpath)/kadupro/scripts/misc.kkp
	source $module(scriptpath)/kadupro/scripts/control.kkp
	source $module(scriptpath)/kadupro/scripts/emotes.kkp
	source $module(scriptpath)/kadupro/scripts/imieniny.kkp
	source $module(scriptpath)/kadupro/scripts/lastseen.kkp
	source $module(scriptpath)/kadupro/scripts/clone.kkp
	source $module(scriptpath)/kadupro/scripts/statusplus.kkp
	source $module(scriptpath)/kadupro/scripts/autoresponder.kkp
	source $module(scriptpath)/kadupro/scripts/board.kkp
	source $module(scriptpath)/kadupro/scripts/post-it.kkp
	source $module(scriptpath)/kadupro/scripts/words_fix.kkp
	source $module(scriptpath)/kadupro/scripts/stats.kkp
	source $module(scriptpath)/kadupro/scripts/list.kkp
	source $module(scriptpath)/kadupro/scripts/addrmem.kkp
	source $module(scriptpath)/kadupro/scripts/help.kkp
	source $module(scriptpath)/kadupro/scripts/tips.kkp
	source $module(scriptpath)/kadupro/scripts/about.kkp


########################################################################
###   creaging main KaduPro menu                                     ###
########################################################################

### This function create a separator in main KaduPro menu
proc separator_create_popup {} {
	global M
	popup addseparator $M(kadupro)
}

### Creating main KaduPro menu:
  set M(kadupro) "Kadu&Pro"
  popup new $M(kadupro)

### and filling it with popups:
### names of functions comes from name of script in which it can be found.
config_create_popup 

separator_create_popup 

clone_create_popup
statusplus_create_popup
autoresponder_create_popup

separator_create_popup

autostatus_create_popup

### googie said that it should only be available at configuration
# observer_create_popup

board_create_popup
postit_create_popup
fix_words_create_popup

separator_create_popup

list_create_popup
stats_create_popup
addrmem_create_popup

separator_create_popup

tips_create_popup
help_create_popup
about_create_popup


########################################################################
###   finishing loading...                                           ###
########################################################################

### Loading configuration
if {[file exists $config_dir/kadupro.conf]} {
    source $config_dir/kadupro.conf
}

### Adding KP popup to Kadu menu bar,
### Display splash-screen (you can disable it!)
if { $showSplash } {
	showSplashScreen
}      									 ;# splash screen


### What does THAT code do ?
qmenu new $M(kadupro)                    ;# main KP menu


after 100 {                              ;# AFTER is required to let Kadu start up faster
    autoresponder_update_fixed_list      ;# autoresponder list
    update_fixed_list                    ;# status+ list
    remakeClonesMenu                     ;# clones list
    setup_postit_timers                  ;# Display post-it notes during startup
    enable_sound_notify                  ;# enable notify_by_sound timer.. ugly workaround :(
    imieniny:showDayNames
    imieniny:CheckForName-Day

    if {$tips_showOnStartup} {
        tipsWindow
    }
}

### Clean up Kadu after KP unload
on unloadscript cleanUp

proc cleanUp {name} {
    if {$name != "kadupro.tcl"} {return}

    set menu [tr {Invite to paint}]
    if {[usermenu exists $menu]} {
        usermenu delete $menu
    }
    set menu [tr {Send mail to...}]
    if {[usermenu exists $menu]} {
        usermenu delete $menu
    }
    set menu [tr {Last seen...}]
    if { [usermenu exists $menu] } {
        usermenu delete $menu
    }

    saveConfig
}

