# --------------------------------
#
# a minimal multi listboxes megawidget
#
# --------------------------------

namespace eval multilist \
{
  namespace export multilist

  # -----------------
  # the constructor
  # -----------------
  proc multilist {w args} \
  {
      variable {}
      # variables
      set ($w:yview) 0
      set ($w:started) 0
      set ($w:resizing) 0
      # options
      set lheight 20
      set theight 20
      set width(1) 20
      for {set i 2} {$i < 10} {incr i} {set width($i) 0}
      set font {Courier -12}
      set lcolor white
      set tcolor gray90
      array set titles {1 1 2 2 3 3 4 4 5 5 6 6 7 7 8 8 9 9}
      foreach {key value} $args \
      {
          switch -glob -- $key \
          {
              -font     { set font $value }
              -lcolor   { set lcolor $value }
              -tcolor   { set tcolor $value }
              -lheight  { set lheight $value }
              -theight  { set theight $value }
              -titles   { array set titles $value }
              -width1   { set width(1) $value }
              -width2   { set width(2) $value }
              -width3   { set width(3) $value }
              -width4   { set width(4) $value }
              -width5   { set width(5) $value }
              -width6   { set width(6) $value }
              -width7   { set width(7) $value }
              -width8   { set width(8) $value }
              -width9   { set width(9) $value }
          }
      }
      set ($w:chwidth) [font measure $font 0]
      set ($w:theight) $theight
      # widgets
      pack [frame $w] -fill both -expand 1
      frame $w.t -cursor sb_h_double_arrow
      frame $w.f -bg beige
      set ty [expr {$theight / 2}]
      foreach i {1 2 3 4 5 6 7 8 9} \
      {
          if {$width($i) == 0} {break}
          set Width [set width($i)]
          set pwidth [expr {$Width * $($w:chwidth)}]
          canvas $w.t.l$i -width $pwidth -height $theight -bg $tcolor -bd 1 -relief raised \
              -highlightthickness 0 -cursor arrow
          $w.t.l$i create text [expr {$pwidth / 2}] $ty -text $titles($i) -font $font
          listbox $w.f.l$i -yscrollc [namespace code [list yscroll $w]] \
              -font $font -width $Width -height $lheight \
              -bd 2 -relief groove -highlightthickness 0 \
              -exportselection 0
      }
      scrollbar $w.vs -command [namespace code [list yview $w]]
      grid $w.t  -column 0 -row 0 -sticky nsew
      grid $w.f  -column 0 -row 1 -sticky nsew
      grid $w.vs -column 1 -row 0 -rowspan 2 -sticky ns
      foreach i {1 2 3 4 5 6 7 8 9} \
      {
          set j [expr $i-1]
          if {![winfo exists $w.f.l[expr $i+1]]} {
              grid $w.t.l$i -column $j -row 0 -padx 1 -sticky ew
              grid $w.f.l$i -column $j -row 1 -sticky ewns
              break
          } else {
              grid $w.t.l$i -column $j -row 0 -padx 1
              grid $w.f.l$i -column $j -row 1 -sticky ns
          }
      }
      grid rowconfigure    $w 1 -weight 1
      grid columnconfigure $w 0 -weight 1
      grid rowconfigure    $w.t 1 -weight 1
      grid columnconfigure $w.t 2 -weight 1
      grid rowconfigure    $w.f 1 -weight 1
      grid columnconfigure $w.f 2 -weight 1
      # bind the select events
      foreach i {1 2 3 4 5 6 7 8 9} \
      {
          if {![winfo exists $w.f.l$i]} {break}
          set list "synchro $w $i"
          for {set j [expr $i+1]} {$j != $i} {incr j} {
              if {![winfo exists $w.f.l$j]} {
                  set j 0
                  continue
              }
              lappend list $j
          }
          bind $w.f.l$i <<ListboxSelect>> [namespace code $list]
      }
      # return ref
      return $w
  }

  # -----------------
  # the scroll procs
  # -----------------
  # called by a listbox
  proc yscroll {w args} \
  {
      if {![winfo exists $w.vs]} { return }
      eval [linsert $args 0 $w.vs set]
      yview $w moveto [lindex [$w.vs get] 0]
  }
  # called by the scroll bar
  proc yview {w args} \
  {
      variable {}
      if {$($w:yview)} { return }
      set ($w:yview) 1
      foreach i {1 2 3 4 5 6 7 8 9} {
          if {![winfo exists $w.f.l$i]} {break}
          eval $w.f.l$i yview $args
      }
      set ($w:yview) 0
  }
  # called by a select event
  proc synchro {w i1 args} \
  {
      set sel [$w.f.l$i1 cursel]
      foreach ii $args {
          $w.f.l$ii selection clear 0 end
      }
      foreach ii $args {
          foreach item $sel { $w.f.l$ii selection set $item }
      }
  }
}
