/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config_dialog.h"
#include "events.h"
#include "tclstring.h"
#include "tcl_scripting.h"
#include "tclmisc.h"

// Event handler for simple no-argument events
#define TCL_EVENT(name, id) \
	void TclEventHandler::name() \
	{ \
		kdebug("TCL Event: name()\n"); \
		QString filter, key; \
		int ret; \
		TclEvent::Iterator it; \
		for ( it = tsess->tclEventList[id].begin(); it != tsess->tclEventList[id].end(); it++ ) \
		{ \
			ret = Tcl_Eval(tsess->tclInt(), (*it).latin1()); \
			if (ret == TCL_ERROR) \
				tsess->errorFromResult(); \
										  \
			tsess->resetResult(); \
		} \
	}

// Event handler for message acknowledgements events
#define TCL_ACK_EVENT(name, id) \
	void TclEventHandler::name(int seq, UinType uin) \
	{ \
		kdebug("TCL Event: name(%d, %d)\n", seq, (uint)uin); \
		QString filter, key, code; \
		int ret; \
		TclEvent::Iterator it; \
		for ( it = tsess->tclEventList[id].begin(); it != tsess->tclEventList[id].end(); it++ ) \
		{ \
			code = *it + " " + QString::number(seq) + " " + QString::number((uint)uin); \
			ret = Tcl_Eval(tsess->tclInt(), code.latin1()); \
			if (ret == TCL_ERROR) \
				tsess->errorFromResult(); \
										  \
			tsess->resetResult(); \
		} \
	}

TclEventHandler::TclEventHandler(TclSession* s)
{
	kdebugf();
	tsess = s;

	connect(gadu, SIGNAL(connecting()), this, SLOT(connecting()));
	connect(gadu, SIGNAL(connected()), this, SLOT(connected()));
	connect(gadu, SIGNAL(error(GaduError)), this, SLOT(connectionError(GaduError)));
	connect(gadu, SIGNAL(disconnected()), this, SLOT(disconnected()));
	connect(gadu, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&, bool)), this, SLOT(userStatusChanged(const UserListElement&, const UserStatus&, bool)));
	connect(gadu, SIGNAL(systemMessageReceived(const QString&)), this, SLOT(systemMessageReceived(const QString&)));
	connect(gadu, SIGNAL(chatMsgReceived0(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived0(UinsList, const QString&, time_t, bool&)));
	connect(gadu, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived1(UinsList, const QString&, time_t, bool&)));
	connect(gadu, SIGNAL(chatMsgReceived2(UinsList, const QString&, time_t)), this, SLOT(chatReceived2(UinsList, const QString&, time_t)));
	connect(gadu, SIGNAL(messageBlocked(int, UinType)), this, SLOT(messageBlocked(int, UinType)));
	connect(gadu, SIGNAL(messageDelivered(int, UinType)), this, SLOT(messageDelivered(int, UinType)));
	connect(gadu, SIGNAL(messageQueued(int, UinType)), this, SLOT(messageQueued(int, UinType)));
	connect(gadu, SIGNAL(messageNotDelivered(int, UinType)), this, SLOT(messageNotDelivered(int, UinType)));
	connect(gadu, SIGNAL(messageAccepted(int, UinType)), this, SLOT(messageAccepted(int, UinType)));
	connect(gadu, SIGNAL(messageRejected(int, UinType)), this, SLOT(messageRejected(int, UinType)));
	connect(gadu, SIGNAL(newSearchResults(SearchResults&, int, int)), this, SLOT(newSearchResults(SearchResults&, int, int)));
	connect(gadu, SIGNAL(userListExported(bool)), this, SLOT(userListExported(bool)));
	connect(gadu, SIGNAL(userListCleared(bool)), this, SLOT(userListCleared(bool)));
	connect(gadu, SIGNAL(userListImported(bool, UserList&)), this, SLOT(userListImported(bool, UserList&)));
	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroyed(const UinsList&)), this, SLOT(chatDestroyed(const UinsList&)));
	connect(&userlist, SIGNAL(userDataChanged(const UserListElement* const, const UserListElement* const, bool)), this, SLOT(userDataChanged(const UserListElement* const, const UserListElement* const, bool)));
	connect(&userlist, SIGNAL(dnsNameReady(UinType)), this, SLOT(userlist_dnsNameReady(UinType)));
	connect(kadu, SIGNAL(changingStatus(UserStatus&, bool&)), this, SLOT(changingStatus(UserStatus&, bool&)));
	connect(kadu, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(keyPressed(QKeyEvent*)));
	connect(this, SIGNAL(searchingFinished(int)), this, SLOT(searchInPubDirFinished(int)));

	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated_Watch(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying_Watch(const UinsList&)));
	for ( uint i = 0; chat_manager->chats().count() > i; i++ )
		manageReadyChats(chat_manager->chats()[i], TRUE);

	ConfigDialog::registerSlotOnCreate(this, SLOT(configOpen()));
	ConfigDialog::registerSlotOnApply(this, SLOT(configApply()));
	ConfigDialog::registerSlotOnClose(this, SLOT(configClose()));
}

TclEventHandler::~TclEventHandler()
{
	kdebugf();
	disconnect(gadu, SIGNAL(connecting()), this, SLOT(connecting()));
	disconnect(gadu, SIGNAL(connected()), this, SLOT(connected()));
	disconnect(gadu, SIGNAL(error(GaduError)), this, SLOT(connectionError(GaduError)));
	disconnect(gadu, SIGNAL(disconnected()), this, SLOT(disconnected()));
	disconnect(gadu, SIGNAL(userStatusChanged(const UserListElement&, const UserStatus&, bool)), this, SLOT(userStatusChanged(const UserListElement&, const UserStatus&, bool)));
	disconnect(gadu, SIGNAL(systemMessageReceived(const QString&)), this, SLOT(systemMessageReceived(const QString&)));
	disconnect(gadu, SIGNAL(chatMsgReceived0(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived0(UinsList, const QString&, time_t, bool&)));
	disconnect(gadu, SIGNAL(chatMsgReceived1(UinsList, const QString&, time_t, bool&)), this, SLOT(chatReceived1(UinsList, const QString&, time_t, bool&)));
	disconnect(gadu, SIGNAL(chatMsgReceived2(UinsList, const QString&, time_t)), this, SLOT(chatReceived2(UinsList, const QString&, time_t)));
	disconnect(gadu, SIGNAL(messageBlocked(int, UinType)), this, SLOT(messageBlocked(int, UinType)));
	disconnect(gadu, SIGNAL(messageDelivered(int, UinType)), this, SLOT(messageDelivered(int, UinType)));
	disconnect(gadu, SIGNAL(messageQueued(int, UinType)), this, SLOT(messageQueued(int, UinType)));
	disconnect(gadu, SIGNAL(messageNotDelivered(int, UinType)), this, SLOT(messageNotDelivered(int, UinType)));
	disconnect(gadu, SIGNAL(messageAccepted(int, UinType)), this, SLOT(messageAccepted(int, UinType)));
	disconnect(gadu, SIGNAL(messageRejected(int, UinType)), this, SLOT(messageRejected(int, UinType)));
	disconnect(gadu, SIGNAL(newSearchResults(SearchResults&, int, int)), this, SLOT(newSearchResults(SearchResults&, int, int)));
	disconnect(gadu, SIGNAL(userListExported(bool)), this, SLOT(userListExported(bool)));
	disconnect(gadu, SIGNAL(userListCleared(bool)), this, SLOT(userListCleared(bool)));
	disconnect(gadu, SIGNAL(userListImported(bool, UserList&)), this, SLOT(userListImported(bool, UserList&)));
	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroyed(const UinsList&)), this, SLOT(chatDestroyed(const UinsList&)));
	disconnect(&userlist, SIGNAL(userDataChanged(const UserListElement* const, const UserListElement* const, bool)), this, SLOT(userDataChanged(const UserListElement* const, const UserListElement* const, bool)));
	disconnect(&userlist, SIGNAL(dnsNameReady(UinType)), this, SLOT(userlist_dnsNameReady(UinType)));
	disconnect(kadu, SIGNAL(changingStatus(UserStatus&, bool&)), this, SLOT(changingStatus(UserStatus&, bool&)));
	disconnect(kadu, SIGNAL(keyPressed(QKeyEvent*)), this, SLOT(keyPressed(QKeyEvent*)));
	disconnect(this, SIGNAL(searchingFinished(int)), this, SLOT(searchInPubDirFinished(int)));

	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated_Watch(const UinsList&)));
	for ( uint i = 0; chat_manager->chats().count() > i; i++ )
		manageReadyChats(chat_manager->chats()[i], FALSE);

	ConfigDialog::unregisterSlotOnCreate(this, SLOT(configOpen()));
	ConfigDialog::unregisterSlotOnApply(this, SLOT(configApply()));
	ConfigDialog::unregisterSlotOnClose(this, SLOT(configClose()));
}

void TclEventHandler::chatCreated_Watch(const UinsList& senders)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(senders);
	connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
	connect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
	connect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
	connect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
	connect(chat->edit, SIGNAL(keyReleased(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyReleased(QKeyEvent*, CustomInput*)));
}

void TclEventHandler::chatDestroying_Watch(const UinsList& senders)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(senders);
	disconnect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
	disconnect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
	disconnect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
	disconnect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
	disconnect(chat->edit, SIGNAL(keyReleased(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyReleased(QKeyEvent*, CustomInput*)));
}

void TclEventHandler::manageReadyChats(Chat* chat, bool conn)
{
	kdebugf();
	if (conn)
	{
		connect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
		connect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
		connect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
		connect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
		connect(chat->edit, SIGNAL(keyReleased(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyReleased(QKeyEvent*, CustomInput*)));
	}
	else
	{
		disconnect(chat, SIGNAL(messageSendRequested(Chat*)), this, SLOT(chatMsgSendReq(Chat*)));
		disconnect(chat, SIGNAL(messageSent(Chat*)), this, SLOT(chatMsgSent(Chat*)));
		disconnect(chat, SIGNAL(messageFiltering(const UinsList&, QCString&, bool&)), this, SLOT(outputMessageFiltering(const UinsList&, QCString&, bool&)));
		disconnect(chat->edit, SIGNAL(keyPressed(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyPressed(QKeyEvent*, CustomInput*)));
		disconnect(chat->edit, SIGNAL(keyReleased(QKeyEvent*, CustomInput*)), this, SLOT(chatKeyReleased(QKeyEvent*, CustomInput*)));
	}
}

/* Tcl Event
chat0 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, ktra jeszcze nie zostaa wywietlona w oknie Chat,
ani dodana do historii. Jeli funkcja obsugujca to zdarzenie zwrci warto logiczn 1,
to zdarzenia chat1 ani chat2 nie nastpi, a uytkownik nie zostanie powiadomiony
o nowej wiadomoci, ani te jej nie zobaczy.
*/
void TclEventHandler::chatReceived0(UinsList senders, const QString& msg, time_t time, bool& grab)
{
	kdebug("TCL Event: chatReceived0(%s)\n", msg.latin1());
	int ret;
	QString filter, code, key, err;
	QStringList uins;

	QValueList<uin_t>::Iterator _it;
	for ( _it = senders.begin(); _it != senders.end(); _it++ )
		uins.append(QString::number(*_it));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[20].begin(); it != tsess->tclEventList[20].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(msg);
		str->appendElement(QString::number(time));

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		QString tmp = Tcl_GetStringResult(tsess->tclInt());
		if (!tmp.isEmpty())
		{
			int retVal;
			tsess->resetResult();
			ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
			if (ret != TCL_ERROR)
				if (retVal)
					grab = TRUE;
		}
		tsess->resetResult();
	}
}

/* Tcl Event
chat1 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, ktra jeszcze nie zostaa wysana do uytkownika Kadu.
Jeli funkcja obsugujca to zdarzenie zwrci warto logiczn 1,
to zdarzenie chat2 nie nastpi, a uytkownik nie zostanie powiadomiony o nowej wiadomoci.
*/
void TclEventHandler::chatReceived1(UinsList senders, const QString& msg, time_t time, bool& grab)
{
	kdebug("TCL Event: chatReceived1(%s)\n", msg.latin1());
	int ret;
	QString filter, code, key, err;
	QStringList uins;

	QValueList<uin_t>::Iterator _it;
	for ( _it = senders.begin(); _it != senders.end(); _it++ )
		uins.append(QString::number(*_it));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[30].begin(); it != tsess->tclEventList[30].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(msg);
		str->appendElement(QString::number(time));

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		QString tmp = Tcl_GetStringResult(tsess->tclInt());
		if (!tmp.isEmpty())
		{
			int retVal;
			tsess->resetResult();
			ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
			if (!chat_manager->findChatByUins(senders))
			{
				if (ret != TCL_ERROR)
					if (retVal)
						grab = TRUE;
			}
		}
		tsess->resetResult();
	}
}

/* Tcl Event
chat2 wysyajce_UINy[lista_liczb] tre[acuch] czas[unixtime]
Otrzymano wiadomo klasy chat, o ktrej uytkownik zosta ju powiadomiony.
*/
void TclEventHandler::chatReceived2(UinsList senders, const QString& msg, time_t time)
{
	kdebug("TCL Event: chatReceived2(%s)\n", msg.latin1());
	int ret;
	QString filter, code, key;
	QStringList uins;

	QValueList<uin_t>::Iterator _it;
	for ( _it = senders.begin(); _it != senders.end(); _it++ )
		uins.append(QString::number(*_it));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[40].begin(); it != tsess->tclEventList[40].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(msg);
		str->appendElement(QString::number(time));

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
chatkey UINy_okna_chat[lista_liczb] klawisz[acuch] modyfikator[acuch] kod_ASCII[liczba]
Wcinito pewien klawisz w oknie chat, ktrego UINy s dane. Jeli wcinity klawisz
jest znakiem, ktry mona wywietli, to argument klawisza bdzie zawiera ten znak,
natomiast jeli jest to klawisz specjalny, np. Home, Shift, czy Strzalka w lewo, wtedy
argument klawisza bedzie zawiera odpowiednio: home, shift, left. Pozostae klawisze
analogicznie. Argument modyfikatora zawiera jeden z trzech modyfikatorw
(<b>alt</b>, <b>control</b>, <b>shift</b>) jeli podczas wcinicia aktualnego klawisza ktry
z tych modyfikatorw by rwnierz trzymany. Niemoliwe jest wystpienie wicej ni jednego
modyfikatora. Jeli dany klawisz nie jest rozpoznawany, to zwrcone zostanie sowo <b>unknown</b>.
Sowo to jest rwnierz zwracane dla klawiszy wciskanych razem z z modyfikatorem <b>control</b>.
W takim przypadku, aby rozezna si, ktry klawisz zosta wciniety, naley posuy si
ostatnim z argumentw - kodem ASCII znaku otrzymanego przez kombinacj <b>control</b> + klawisz.
Przykadowo, dla kombinacji <b>ctrl+a</b> byaby to liczba 1, dla <b>ctrl+d</b> 4, itd.
*/
void TclEventHandler::chatKeyPressed(QKeyEvent* e, CustomInput* k)
{
	int ret;
	QString filter, code, key, keystr = e->text(), modstr = "", ascii = QString::number(e->ascii());
	Chat* chat = (Chat*)k->parentWidget()->parentWidget()->parentWidget();
	UinsList senders = chat->uins();

	tclMisc->getKey(e, keystr, modstr);

	QStringList uins;
	QValueList<uin_t>::Iterator _it;
	for ( _it = senders.begin(); _it != senders.end(); _it++ )
		uins.append(QString::number(*_it));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[50].begin(); it != tsess->tclEventList[50].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(keystr);
		str->appendElement(modstr);
		str->appendElement(ascii);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
chatkeyrel UINy_okna_chat[lista_liczb] klawisz[acuch] modyfikator[acuch] kod_ASCII[liczba]
Zdarzenie analogiczne do <b>chatkey</b>, z t rnic, e dotyczy zwolnienia danego klawisza.
*/
void TclEventHandler::chatKeyReleased(QKeyEvent* e, CustomInput* k)
{
	int ret;
	QString filter, code, key, keystr = e->text(), modstr = "", ascii = QString::number(e->ascii());
	Chat* chat = (Chat*)k->parentWidget()->parentWidget()->parentWidget();
	UinsList senders = chat->uins();

	tclMisc->getKey(e, keystr, modstr);

	QStringList uins;
	QValueList<uin_t>::Iterator _it;
	for ( _it = senders.begin(); _it != senders.end(); _it++ )
		uins.append(QString::number(*_it));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[52].begin(); it != tsess->tclEventList[52].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(keystr);
		str->appendElement(modstr);
		str->appendElement(ascii);

		ret = tsess->eval(str);
		delete str;
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
chmystatus status[acuch] opis[acuch]
Uytkownik Kadu zmienia status. Jeli funkcja obsugujca to zdarzenie
zwrci warto 1, to status ten nie zostanie ustawiony mimo wszystko.
*/
void TclEventHandler::changingStatus(UserStatus& status, bool& stop)
{
	kdebug("TCL Event: changingStatus(%d, %s, %d)\n", status.index(), status.description().latin1(), stop);
	int ret;
	QString filter, code, key, err,
		stat = tclMisc->KaduStatusToTclStatus(status.index());

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[60].begin(); it != tsess->tclEventList[60].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(stat);
		str->appendElement(status.description());

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		QString tmp = Tcl_GetStringResult(tsess->tclInt());
		if (!tmp.isEmpty())
		{
			int retVal;
			tsess->resetResult();
			ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
			if (ret != TCL_ERROR)
				if (retVal)
					stop = TRUE;
		}
		tsess->resetResult();
	}
}

/* Tcl Event
chstatus uin[liczba] stary_status[acuch] status[acuch] opis[acuch]
Ktry z kontaktw zmieni swj status.
*/
void TclEventHandler::userStatusChanged(const UserListElement &ule, const UserStatus &oldStatus, bool onConnection)
{
	int ret;
	QString filter, code, status, key;

	kdebug("TCL Event: userStatusChanged(%d, %d, %d)\n", ule.uin(), oldStatus.index(), ule.status().index());

	status = tclMisc->KaduStatusToTclStatus(ule.status().index());
	if (status.isEmpty())
		tsess->error(tr("Unknown status type %1 in event chstatus.").arg(status));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[70].begin(); it != tsess->tclEventList[70].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(QString::number(ule.uin()));
		str->appendElement(tclMisc->KaduStatusToTclStatus(oldStatus.index()));
		str->appendElement(status);
		str->appendElement(ule.status().description());

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
closedchat UINy[lista_liczb]
Zostao zaknite okno rozmowy.
*/
void TclEventHandler::chatDestroyed(const UinsList& senders)
{
	int ret;
	QStringList uins;
	QString filter, code, key;
	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	kdebug("TCL Event: chatDestroyed(%s)\n", uins.join(" ").latin1());
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[80].begin(); it != tsess->tclEventList[80].end(); it++ )
	{
		code = *it + " \{" + uins.join(" ") + "}";
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
closingchat UINy[lista_liczb]
Zostao zaknite okno rozmowy.
*/
void TclEventHandler::chatDestroying(const UinsList& senders)
{
	int ret;
	QStringList uins;
	QString filter, code, key;
	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	kdebug("TCL Event: chatDestroying(%s)\n", uins.join(" ").latin1());
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[90].begin(); it != tsess->tclEventList[90].end(); it++ )
	{
		code = *it + " \{" + uins.join(" ") + "}";
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
configopen
Otwarto okno konfiguracji Kadu.
*/
TCL_EVENT(configOpen, 94);

/* Tcl Event
configapply
Wcinito przycisk "Zastosuj" w oknie konfiguracji Kadu.
*/
TCL_EVENT(configApply, 95);

/* Tcl Event
configclose
Zamknito okno konfiguracji Kadu.
*/
TCL_EVENT(configClose, 96);

/* Tcl Event
connected
Nawizano poczenie z serwerem.
*/
TCL_EVENT(connected, 100);

/* Tcl Event
connecting
Rozpoczto procedur czenia z serwerem.
*/
TCL_EVENT(connecting, 110);

/* Tcl Event
connerr bd[acuch]
Bd poczenia z serwerem.
*/
void TclEventHandler::connectionError(GaduError why)
{
	kdebug("TCL Event: connectionError(%d)\n", why);
	int ret;
	QString filter, code, key, string_why;

	string_why = tclMisc->GaduErrorToString(why);

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[120].begin(); it != tsess->tclEventList[120].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(string_why);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
disconnected
Otrzymano polecenie rozczenia.
*/
TCL_EVENT(disconnected, 130);

/* Tcl Event
key klawisz[acuch] modyfikator[acuch] kod_ASCII[liczba]
Wcisnito pewien klawisz w gwnym oknie Kadu. Argumenty analogiczne
do tych ze zdarzenia <b>chatkey</b>.
*/
void TclEventHandler::keyPressed(QKeyEvent* e)
{
	int ret;
	QString filter, code, key, keystr = e->text(), modstr = "", ascii = QString::number(e->ascii());

	tclMisc->getKey(e, keystr, modstr);

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[136].begin(); it != tsess->tclEventList[136].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(keystr);
		str->appendElement(modstr);
		str->appendElement(ascii);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
msg_blocked identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze zostaa zablokowana przez serwer.
*/
TCL_ACK_EVENT(messageBlocked, 152);

/* Tcl Event
msg_delivered identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze zostaa dostarczone.
*/
TCL_ACK_EVENT(messageDelivered, 153);

/* Tcl Event
msg_queued identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze zostaa ustawiona w kolejce.
*/
TCL_ACK_EVENT(messageQueued, 154);

/* Tcl Event
msg_boxfull identyfikator[liczba] UINy[lista_liczb]
Kolejka dla danego(-ych) UINu(-w) jest zapeniona.
*/
TCL_ACK_EVENT(messageBoxFull, 155);

/* Tcl Event
msg_notdelivered identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze <U>nie</U> zostaa dostarczona.
*/
TCL_ACK_EVENT(messageNotDelivered, 156);

/* Tcl Event
msg_accepted identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze zostaa zaakceptowana przez serwer.
*/
TCL_ACK_EVENT(messageAccepted, 157);

/* Tcl Event
msg_rejected identyfikator[liczba] UINy[lista_liczb]
Wiadomo o danym identyfikatorze zostaa odrzucona przez serwer.
*/
TCL_ACK_EVENT(messageRejected, 158);

/* Tcl Event
msgsent UINy[lista_liczb]
Wiadomo z okna rozmowy z tym(i) UINem(ami) zosta ju wysana,
ale nie otrzymano jeszcze potwierdzenia, po prostu poszo w wiat.
*/
void TclEventHandler::chatMsgSent(Chat* chat)
{
	int ret;
	UinsList senders = chat->uins();
	QStringList uins;
	QString filter, code, key;
	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	kdebug("TCL Event: chatMsgSendReq(%s)\n", uins.join(" ").latin1());
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[160].begin(); it != tsess->tclEventList[160].end(); it++ )
	{
		code = *it + " \{" + uins.join(" ") + "}";
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
newchat UINy[lista_liczb]
Zostao otwarte nowe okno rozmowy.
*/
void TclEventHandler::chatCreated(const UinsList& senders)
{
	int ret;
	QStringList uins;
	QString filter, code, key;
	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	kdebug("TCL Event: chatCreated(%s)\n", uins.join(" ").latin1());
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[170].begin(); it != tsess->tclEventList[170].end(); it++ )
	{
		code = *it + " \{" + uins.join(" ") + "}";
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
progresscancel
Anulowano okno postpu.
*/
TCL_EVENT(msgBoxCancel, 180);

/* Tcl Event
results ID[liczba] szukany(a)_UIN[liczba] pseudonim[acuch] imi[acuch] urodziny[data] miasto[acuch] status[acuch]
Otrzymano z serwera rezultat operacji wyszukiwania w katalogu publicznym. <i>ID</i> jest identyfikatorem
przeszukiwania katalogu publicznego. Komenda <b>search</b> zwraca taki identyfikator w celu dopasowania,
czy otrzymane rezultaty wyszukiwania s tymi, o ktre pytana ww. komend.
*/
void TclEventHandler::newSearchResults(SearchResults& res, int seq, int lastUin)
{
	kdebug("TCL Event: newSearchResults()\n");
	QString filter, code, key, status_str;
	SearchResults::iterator searchIterator;
	int count = res.count(), ret, status;

	if (count > 0)
	{
		for (int i = 0; i < count; i++)
		{
			status = res[i].Stat.toStatusNumber() & 127;
			status_str = tclMisc->GGStatusToTclStatus(status);
			if (status_str.isEmpty())
				tsess->error(tr("Status type in event 'newSearchResult' is unknown."));

			TclEvent::Iterator it;
			for ( it = tsess->tclEventList[190].begin(); it != tsess->tclEventList[190].end(); it++ )
			{
				TclString* str = new TclString();

				str->appendElement(*it);
				str->appendElement(QString::number(seq));
				str->appendElement(res[i].Uin);
				str->appendElement(res[i].Nick);
				str->appendElement(res[i].First);
				str->appendElement(res[i].Born);
				str->appendElement(res[i].City);
				str->appendElement(status_str);

				ret = tsess->eval(str);
				delete str;

				if (ret == TCL_ERROR)
					tsess->errorFromResult();

				tsess->resetResult();
			}
		}
	}
	checkForNext(res, seq, lastUin, count);
}

/* Tcl Event
searchdone ID[liczba]
Zakoczono wyszukiwanie dla danego <i>identyfikatora</i> zwrconego przez komend <b>search</b>.
*/
void TclEventHandler::searchInPubDirFinished(int id)
{
	kdebug("TCL Event: searchingFinished(%d))\n", id);
	int ret;
	QString filter, code, key;

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[200].begin(); it != tsess->tclEventList[200].end(); it++ )
	{
		code = *it + " " + QString::number(id);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
searchnext ID[liczba] pozycja[liczba]
To zdarzenie podaje numer <i>pozycji</i>, od ktrej naley rozpocz dalsze przeszukiwanie (o ile potrzeba)
katalogu publicznego dla podanego <i>identyfikatora</i> szukania (zwrconego przez komend <b>search</b>).
Wystpuje ono jedynie wtedy, gdy zwrcone wyniki nie s wszystkimi moliwymi (w katalogu jest wicej ni 20
pasujcych odpowiedzi). Numer pozycji jest argumentem dla opcji <b>-from</b> komendy <b>search</b>.
Jeli ju wszystkie odpowiedzi zostay odebrane, to <i>pozycja</i> bdzie wynosi 0.
*/
void TclEventHandler::checkForNext(SearchResults& res, int seq, int lastUin, int count)
{
	int ret;
	QString filter, code, key;
	if (tclPubDirList.keys().findIndex(seq) == -1)
		return;

	if (count < 1 || lastUin == tclPubDirList[seq]->FromUin)
	{
		delete tclPubDirList[seq];
		tclPubDirList.remove(seq);
		emit searchingFinished(seq);
		return;
	}
	delete tclPubDirList[seq];
	tclPubDirList.remove(seq);

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[210].begin(); it != tsess->tclEventList[210].end(); it++ )
	{
		code = *it + " " + QString::number(seq) + " " + QString::number(lastUin);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
sendmsg1 UINy[lista_liczb]
Uytkownik wcisn Enter, rzdajc tym samym wysania wiadomoci do
UINa(ow). Jest to odpowiedni moment na edycj tekstu, przed wysaniem do serwera.
*/
void TclEventHandler::chatMsgSendReq(Chat* chat)
{
	int ret;
	UinsList senders = chat->uins();
	QStringList uins;
	QString filter, code, key;
	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	kdebug("TCL Event: chatMsgSendReq(%s)\n", uins.join(" ").latin1());
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[220].begin(); it != tsess->tclEventList[220].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
sendmsg2 UINy[lista_liczb] wiadomo[acuch]
Zdarzenie wywoywane tu przed wysaniem, <i>wiadomoci</i> do <i>UINw</i>, ju w jej docelowej
postaci. Jeli procedura obsugujca to zdarzenie zwrci warto 1, to wiadomo nie zostanie
ostatecznie wysana.
*/
void TclEventHandler::outputMessageFiltering(const UinsList& senders, QCString& msg, bool &stop)
{
	kdebug("TCL Event: outputMessageFiltering(%s)\n", (const char*)msg);
	int ret;
	const char* msg_c = msg;
	QString filter, code, key, err, _msg = cp2unicode((const unsigned char*)msg_c);
	QStringList uins;

	for ( uint i = 0; senders.count() > i; i++ )
		uins.append(QString::number(senders[i]));

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[230].begin(); it != tsess->tclEventList[230].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(uins.join(" "));
		str->appendElement(_msg);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		QString tmp = Tcl_GetStringResult(tsess->tclInt());
		if (!tmp.isEmpty())
		{
			int retVal;
			tsess->resetResult();
			ret = Tcl_GetBoolean(tsess->tclInt(), tmp.latin1(), &retVal);
			if (ret != TCL_ERROR)
				if (retVal)
					stop = TRUE;
		}
		tsess->resetResult();
	}
}

/* Tcl Event
sysmsg tre[acuch]
Otrzymano jak wiadomo z serwera GG.
*/
void TclEventHandler::systemMessageReceived(const QString &msg)
{
	kdebug("TCL Event: systemMessageReceived(%s)\n", msg.latin1());
	int ret;
	QString filter, code, key;

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[150].begin(); it != tsess->tclEventList[150].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(msg);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
unloadmodule
Modu TCL jest wyadowywany.
*/
TCL_EVENT(unloadModule, 240);

/* Tcl Event
unloadscript nazwa[acuch]
Skrypt o podanej <i>nazwie</i> (pliku) jest wyadowywany. To zdarzenie jest wywoywane
zarwno, gdy skrypt jest wyadowywany w oknie adowania skryptw moduu Tcl, jak i przy
wyadowywaniu caego moduu Tcl. Przydatne zdarzenie do "posprztania" po skrypcie.
*/
void TclEventHandler::scriptUnload(QString name)
{
	kdebug("TCL Event: scriptUnload(%s)\n", name.latin1());
	int ret;
	QString filter, code, key;

	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[245].begin(); it != tsess->tclEventList[245].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);
		str->appendElement(name);

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
usl Stary_Kontakt[lista_danych] Nowy_Kontakt[lista_danych]
Dokonano zmian na licie kontaktw. Kady z dwch argumentw
jest list TCL, ktrej elementy stanowi kolejno: UIN,
nazw alternatywn (ta wywietlana), pseudonim, imi,
nazwisko, numer telefonu, email, grupa(-y), anonimowy,
blokowany, niewidoczny dla, powiadamiaj. Cztery ostatnie
elementy maj wartoci logiczne - 1/0, pierwszy jest liczb,
pozostae za acuchami. Jeli nowy kontakt jest dodawany
to pierwsza lista zawiera 0 elementw, natomiast gdy kontakt
jest usuwany, wtedy druga lista ma 0 elementw.
*/
void TclEventHandler::userDataChanged(const UserListElement* const oldusl, const UserListElement* const newusl, bool)
{
	kdebug("TCL Event: userlistDataChanged()\n");
	int ret;
	QString filter, code, key;
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[250].begin(); it != tsess->tclEventList[250].end(); it++ )
	{
		TclString* str = new TclString();

		str->appendElement(*it);

		str->listBegin();
		if (oldusl != NULL)
		{
			str->appendElement(QString::number(oldusl->uin()));
			str->appendElement(oldusl->altNick());
			str->appendElement(oldusl->nickName());
			str->appendElement(oldusl->firstName());
			str->appendElement(oldusl->lastName());
			str->appendElement(oldusl->mobile());
			str->appendElement(oldusl->email());
			str->appendElement(oldusl->group());
			str->appendElement(QString::number(oldusl->isAnonymous()));
			str->appendElement(QString::number(oldusl->blocking()));
			str->appendElement(QString::number(oldusl->offlineTo()));
			str->appendElement(QString::number(oldusl->notify()));
		}
		str->listEnd();

		str->listBegin();
		if (newusl)
		{
			str->appendElement(QString::number(newusl->uin()));
			str->appendElement(newusl->altNick());
			str->appendElement(newusl->nickName());
			str->appendElement(newusl->firstName());
			str->appendElement(newusl->lastName());
			str->appendElement(newusl->mobile());
			str->appendElement(newusl->email());
			str->appendElement(newusl->group());
			str->appendElement(QString::number(newusl->isAnonymous()));
			str->appendElement(QString::number(newusl->blocking()));
			str->appendElement(QString::number(newusl->offlineTo()));
			str->appendElement(QString::number(newusl->notify()));
		}
		str->listEnd();

		ret = tsess->eval(str);
		delete str;

		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
usldns UIN[liczba]
DNS dla UIN'a jest gotowy.
*/
void TclEventHandler::userlist_dnsNameReady(UinType _uin)
{
	kdebug("TCL Event: userlist_dnsNameReady(%d)\n", _uin);
	int ret;
	QString filter, code, key;
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[260].begin(); it != tsess->tclEventList[260].end(); it++ )
	{
		code = *it + " " + QString::number(_uin);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
uslexp rezultat[warto_logiczna]
Operacja eksportowania listy uytkownik zostaa zakoczona. Rezultat przyjmuje
warto 1, gdy operacja zakoczya si sukcesem, w przeciwnym wypadku przyjmuje warto 0.
*/
void TclEventHandler::userListExported(bool ok)
{
	kdebug("TCL Event: userListExported(%d)\n", ok);
	int ret;
	QString filter, code, key;
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[270].begin(); it != tsess->tclEventList[270].end(); it++ )
	{
		code = *it + " " + QString::number(ok);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
uslclr rezultat[warto_logiczna]
Operacja czyszczenia listy uytkownik zostaa zakoczona. Rezultat przyjmuje
warto 1, gdy operacja zakoczya si sukcesem, w przeciwnym wypadku przyjmuje warto 0.
*/
void TclEventHandler::userListCleared(bool ok)
{
	kdebug("TCL Event: userListCleared(%d)\n", ok);
	int ret;
	QString filter, code, key;
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[280].begin(); it != tsess->tclEventList[280].end(); it++ )
	{
		code = *it + " " + QString::number(ok);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}

/* Tcl Event
uslimp rezultat[warto_logiczna]
Operacja importowania listy uytkownik zostaa zakoczona. Rezultat przyjmuje
warto 1, gdy operacja zakoczya si sukcesem, w przeciwnym wypadku przyjmuje warto 0.
*/
void TclEventHandler::userListImported(bool ok, UserList& usl)
{
	kdebug("TCL Event: userListImported(%d)\n", ok);
	int ret;
	QString filter, code, key;
	TclEvent::Iterator it;
	for ( it = tsess->tclEventList[290].begin(); it != tsess->tclEventList[290].end(); it++ )
	{
		code = *it + " " + QString::number(ok);
		ret = Tcl_Eval(tsess->tclInt(), code.latin1());
		if (ret == TCL_ERROR)
			tsess->errorFromResult();

		tsess->resetResult();
	}
}
