#ifndef EVENTS_H
#define EVENTS_H

#include <qobject.h>
#include "gadu.h"
#include "userlist.h"

class TclSession;
class Chat;
class QKeyEvent;
class CustomInput;

class TclEventHandler : public QObject
{
	Q_OBJECT

	public:
		TclEventHandler(TclSession*);
		~TclEventHandler();
		void unloadModule();
		void checkForNext(SearchResults&, int, int, int);

	private slots:
		void connected();
		void connecting();
		void disconnected();
		void connectionError(GaduError);
		void userStatusChanged(const UserListElement&, const UserStatus&, bool);
		void systemMessageReceived(const QString&);
		void chatReceived0(UinsList senders, const QString& msg, time_t time, bool& grab);
		void chatReceived1(UinsList senders, const QString& msg, time_t time, bool& grab);
		void chatReceived2(UinsList senders, const QString& msg, time_t time);
		void messageBlocked(int seq, UinType uin);
		void messageDelivered(int seq, UinType uin);
		void messageQueued(int seq, UinType uin);
		void messageBoxFull(int seq, UinType uin);
		void messageNotDelivered(int seq, UinType uin);
		void messageAccepted(int seq, UinType uin);
		void messageRejected(int seq, UinType uin);
		void newSearchResults(SearchResults&, int seq, int lastUin);
		void chatCreated(const UinsList&);
		void chatDestroying(const UinsList&);
		void chatDestroyed(const UinsList&);
		void userDataChanged(const UserListElement* const, const UserListElement* const, bool);
		void userlist_dnsNameReady(UinType);
		void userListExported(bool ok);
		void userListCleared(bool ok);
		void userListImported(bool ok, UserList& usl);
		void chatMsgSendReq(Chat*);
		void chatMsgSent(Chat*);
		void chatKeyPressed(QKeyEvent*, CustomInput*);
		void chatKeyReleased(QKeyEvent*, CustomInput*);
		void searchInPubDirFinished(int);
		void changingStatus(UserStatus&, bool&);
		void outputMessageFiltering(const UinsList&, QCString&, bool&);
		void chatCreated_Watch(const UinsList&);
		void chatDestroying_Watch(const UinsList&);
		void keyPressed(QKeyEvent*);

	public slots:
		void msgBoxCancel();
		void configOpen();
		void configApply();
		void configClose();
		void scriptUnload(QString name);

	signals:
		void searchingFinished(int);

	private:
		TclSession* tsess;
		void manageReadyChats(Chat*, bool);
};

#endif
