/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Kadu includes
#include "message_box.h"
#include "modules.h"
#include "misc.h"

// TCL module includes
#include "events.h"
#include "commands.h"
#include "tclconsole.h"
#include "tcllineedit.h"
#include "tcltimer.h"
#include "tclloaddialog.h"
#include "tclstring.h"
#include "tclmisc.h"
#include "tcl_scripting.h"

// QT includes
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qcombobox.h>

// Version
#define MODULE_TCL_VERSION "0.6.2-Josephine"
#define MODULE_TCL_RELEASE "20051130"

// Initalisation of module
TclSessions tclSess;
TclLoadDialog *loadDialog;
QTimer* events_loop;
PubDirId tclPubDirList;
TclMisc *tclMisc;

extern "C" int tcl_scripting_init()
{
	kdebug("Initializing TCL module.");
	loadDialog = new TclLoadDialog();
	new TclSession(TRUE);
	tclMisc = new TclMisc();

	QString path = ggPath(".module_tcl_scripts");

	QFile file(path);
	if (file.exists())
	{
		if (file.open( IO_ReadOnly ))
		{
			QTextStream stream(&file);
			QString line;
			while (!stream.atEnd())
			{
				line = stream.readLine();
				if (!line.isEmpty())
				{
					TclSession* tsess = new TclSession();
					if (tsess->load(line) == TCL_ERROR)
					{
						tsess->error(QString::fromUtf8(Tcl_GetStringResult(tsess->tclInt())), TRUE);
						delete tsess;
					}
					else
					{
						QString name = QStringList::split("/", line).last();
						if (!loadDialog->loaded(name))
						{
							loadDialog->list()->insertItem(name);
							for ( uint i = 0; i < loadDialog->lList()->count(); i++ )
							{
								if (loadDialog->lList()->text(i) == name)
									loadDialog->lList()->removeItem(i);
							}
							for ( uint i = 0; i < loadDialog->gList()->count(); i++ )
							{
								if (loadDialog->gList()->text(i) == name)
									loadDialog->gList()->removeItem(i);
							}
						}
					}
				}
			}
			file.close();
		}
	}
	events_loop = new QTimer(0);
	QObject::connect(events_loop, SIGNAL(timeout()), tclSess[0], SLOT(processOneEvent()) );
	events_loop->start( 1, FALSE );

	return 0;
}

extern "C" void tcl_scripting_close()
{
	kdebug("Closing TCL module.\n");

	events_loop->stop();
	QObject::disconnect(events_loop, SIGNAL(timeout()), tclSess[0], SLOT(processOneEvent()) );
	delete events_loop;
	delete tclMisc;

	QString path = ggPath(".module_tcl_scripts");
	QFile file(path);
	if ( file.open( IO_WriteOnly ) )
	{
		QTextStream stream(&file);
		TclSessions::Iterator it = tclSess.begin();
		for ( ++it ; it != tclSess.end(); it++ )
		{
			stream << it.data()->scriptName() << endl;
		}
		file.close();
	}
	else
	{
		QMessageBox::warning( 0, QObject::tr("Warning!"), QObject::tr("Unable to open file %1 for writting. Can't save scripts list.").arg(path) );
	}
	for ( int i = tclSess.count() - 1 ; i > -1 ; i-- )
	{
		tclSess[i]->tclEvtHnd()->unloadModule();
		delete tclSess[i];
	}

	delete loadDialog;
}


// Implementation of TclSession class

TclSession::TclSession(bool main)
{
	kdebugf();

	int i = 0;
	while (tclSess.contains(i))
		i++;

	allowErrors = TRUE;
	setID(i);
	tclSess[i] = this;
	idPtr = new int(i);

	tclrun = TRUE;
	isMain = FALSE;
	tclEventHandler = new TclEventHandler(this);
	start();

	if (main)
	{
		isMain = TRUE;
		Tcl_cons = new TclConsole();
		tclMenu = new QPopupMenu(kadu);
		addMenu();
		Tcl_Eval(tcl_int, "fconfigure stdout -buffering none");
		if (load() == TCL_ERROR)
		{
			for ( QMap<QString,int>::Iterator it = tclMenuBar.begin(); it != tclMenuBar.end(); it++ )
				kadu->menuBar()->removeItem(*it);

			error(QString::fromUtf8(Tcl_GetStringResult(tcl_int)), TRUE);
		}
	}
}

TclSession::~TclSession()
{
	kdebugf();

	for ( QMap<QString,int>::Iterator it = tclMenuBar.begin(); it != tclMenuBar.end(); it++ )
		kadu->menuBar()->removeItem(*it);

	stop();
	if (isMain)
	{
		delMenu();
		delete Tcl_cons;
		delete tclMenu;
	}
	delete tclEventHandler;
	delete idPtr;
	tclSess.remove(sessID);
}

void TclSession::openConsole()
{
	kdebugf();
	if (Tcl_cons->isShown())
	{
		Tcl_cons->raise();
	}
	else
	{
		Tcl_cons->show();
	}
	Tcl_cons->getCmdLine()->setFocus();
}

void TclSession::openLoadDialog()
{
	kdebugf();
	loadDialog->show();
}

void TclSession::addMenu()
{
	kdebugf();
	tclMenu->insertItem(icons_manager.loadIcon("ResendUserlist"), tr("Load dialog"), this, SLOT(openLoadDialog()), 0, -1, 0);
	tclMenu->insertItem(icons_manager.loadIcon("AutoSendMessage"), tr("Open console"), this, SLOT(openConsole()), 0, -1, 1);
	tclMenu->insertSeparator(2);
	if (!config_file.readEntry("TCL", "hidemenu").compare("true"))
	{
		tclMenu->insertItem(icons_manager.loadIcon("SendMessage"), tr("Show TCL menu"), this, SLOT(showTclMenu()), 0, -1, 3);
		menu_id = kadu->mainMenu()->insertItem(tr("&TCL module"), tclMenu, -1, 16);
	}
	else
	{
		tclMenu->insertItem(icons_manager.loadIcon("CancelMessage"), tr("Hide TCL menu"), this, SLOT(hideTclMenu()), 0, -1, 3);
		menu_id = kadu->menuBar()->insertItem(tr("&TCL module"), tclMenu, -1, 1);
	}
	tclMenu->insertItem(icons_manager.loadIcon("History"), tr("About module"), this, SLOT(aboutModule()), 0, -1, 4);
}

void TclSession::aboutModule()
{
	kdebugf();
	QString info;
	info += tr("<b>Module TCL v%1</b><br>");
	info += tr("This module allows to use TCL scripting language under Kadu.") + "<br>";
	info += tr("Home page of project is:") + " ";
	info += "<a href=\"http://scripts.one.pl/tcl4kadu\">http://scripts.one.pl/tcl4kadu</a><br><br>";
	info += tr("Author is Pawel (Googie) Salawa") + " ";
	info += "(<a href=\"mailto:boogie@scripts.one.pl\">boogie@scripts.one.pl</a>).";
	MessageBox::msg(info.arg(MODULE_TCL_VERSION));
}

void TclSession::menuItemSetEnabled(int idx, bool enabled)
{
	kdebugf();
	tclMenu->setItemEnabled(tclMenu->idAt(idx), enabled);
}

void TclSession::delMenu()
{
	kdebugf();
	int itemId = -1;
	for ( uint i = 0; i <= kadu->mainMenu()->count(); i++ )
	{
		if ( ! kadu->mainMenu()->text(kadu->mainMenu()->idAt(i)).compare(tr("&TCL module")) )
		{
			itemId = kadu->mainMenu()->idAt(i);
		}
	}
	if (itemId != -1)
		kadu->mainMenu()->removeItem(menu_id);
	else
		kadu->menuBar()->removeItem(menu_id);

	TclPopups::Iterator it;
	for ( it = tclPopups.begin(); it != tclPopups.end(); it++ )
	{
		kadu->menuBar()->removeItem(it.data().id);
	}
}

void TclSession::hideTclMenu()
{
	kdebugf();
	kadu->menuBar()->removeItem(menu_id);
	menu_id = kadu->mainMenu()->insertItem(tr("&TCL module"), tclMenu, -1, 17);
	tclMenu->removeItemAt(3);
	tclMenu->insertItem(icons_manager.loadIcon("SendMessage"), tr("Show TCL menu"), this, SLOT(showTclMenu()), 0, -1, 3);
	config_file.writeEntry("TCL", "hidemenu", "true");
}

void TclSession::showTclMenu()
{
	kdebugf();
	kadu->mainMenu()->removeItem(menu_id);
	menu_id = kadu->menuBar()->insertItem(tr("&TCL module"), tclMenu, -1, 1);
	tclMenu->removeItemAt(3);
	tclMenu->insertItem(icons_manager.loadIcon("CancelMessage"), tr("Hide TCL menu"), this, SLOT(hideTclMenu()), 0, -1, 3);
	config_file.writeEntry("TCL", "hidemenu", "false");
}

bool TclSession::start()
{
	kdebugf();
	QString arg = qApp->argv()[0];

	// Starting of TCL interpreter
	Tcl_FindExecutable(arg.latin1());
	tcl_int = Tcl_CreateInterp();
	kdebug("Tcl_Init(): Initializing TCL interpreter.\n");
	if (Tcl_Init(tcl_int) == TCL_ERROR)
	{
		error(tr("Error while initializing TCL intepreter:\n%1").arg(Tcl_GetStringResult(tcl_int)));
		return TCL_ERROR;
	}

	// Adding new commands
	addTclCommands();

	// Setting command line arguments
	// argv0
	Tcl_SetVar(tcl_int, "argv0", arg.latin1(), TCL_GLOBAL_ONLY);

	// argv
	arg = qApp->argv()[1];
	for ( int i = 2; i < qApp->argc(); i++ )
	{
		arg.append(" ");
		arg.append(qApp->argv()[i]);
	}
	Tcl_SetVar(tcl_int, "argv", arg.latin1(), TCL_GLOBAL_ONLY);

	// argc
	int c = qApp->argc() - 1;
	Tcl_SetVar(tcl_int, "argc", (char*)QString::number(c).latin1(), TCL_GLOBAL_ONLY);

	// Setting some useful global variables
	QString dir = ggPath("");

	QStringList l = QStringList::split("-", MODULE_TCL_VERSION);
	Tcl_SetVar(tcl_int, "module(version)", l[0].latin1(), TCL_GLOBAL_ONLY);
	Tcl_SetVar(tcl_int, "module(codename)", l[1].latin1(), TCL_GLOBAL_ONLY);
	Tcl_SetVar(tcl_int, "module(release)", MODULE_TCL_RELEASE, TCL_GLOBAL_ONLY);

	Tcl_SetVar(tcl_int, "kadu(datadir)", dataPath().latin1(), TCL_GLOBAL_ONLY);
	Tcl_SetVar(tcl_int, "kadu(configdir)", dir.latin1(), TCL_GLOBAL_ONLY);
	Tcl_SetVar(tcl_int, "kadu(version)", QString(VERSION).latin1(), TCL_GLOBAL_ONLY);

	Tcl_SetVar(tcl_int, "gg(nick)", config_file.readEntry("General", "Nick"), TCL_GLOBAL_ONLY);
	Tcl_SetVar(tcl_int, "gg(uin)", config_file.readEntry("General", "UIN"), TCL_GLOBAL_ONLY);

	return TCL_OK;
}

int TclSession::load(QString file)
{
	kdebugf();
	if (tclrun && QFile::exists(file))
	{
		QStringList tmp = QStringList::split("/", file);
		tmp.remove(tmp.last());
		QString path = "/" + tmp.join("/");
		Tcl_SetVar(tcl_int, "module(scriptpath)", path.latin1(), TCL_GLOBAL_ONLY);

		int status = Tcl_EvalFile(tcl_int, file);
		scriptFile = file;
		tclSess[0]->console()->context()->insertItem(QStringList::split("/", file).last());

		return status;
	}
	else
	{
		return TCL_ERROR;
	}
}

int TclSession::load()
{
	kdebugf();
	QString path = ggPath("default.tcl");
	if (QFile::exists(path))
		return load(path);
	else
		return TCL_OK;
}

void TclSession::stop()
{
	kdebugf();
	if (tclrun)
	{
		QString file = QStringList::split("/", scriptName()).last();
		tclEventHandler->scriptUnload(file);

		killAllTimers();
		Tcl_DeleteInterp(tcl_int);

		int i = 0;
		while ( i < tclSess[0]->console()->context()->count()
			&& file != tclSess[0]->console()->context()->text(i) )
			i++;

		if (i < tclSess[0]->console()->context()->count())
			tclSess[0]->console()->context()->removeItem(i);

		tclrun = FALSE;
	}
}

bool TclSession::isrun()
{
	kdebugf();
	return tclrun;
}

void TclSession::error(QString cmd, bool line)
{
	kdebug("void TclSession::error(QString, bool): \"%s\", in line %d.\n", cmd.latin1(), tcl_int->errorLine);
	if (!allowErrors)
		return;

	QString error = QString(tr("Error in TCL script:\n%1")).arg(cmd);
	if (line)
	{
		error.append("\n");
		error.append(tr("In line:"));
		error.append(" ");
		error.append(QString::number(tcl_int->errorLine));
	}
	MessageBox::msg(error);
	allowErrors = FALSE;
	QTimer::singleShot(2000, this, SLOT(unlockErrors()));
}

void TclSession::errorFromResult()
{
	kdebugf();
	error(QString::fromUtf8(Tcl_GetStringResult(tcl_int)));
}

void TclSession::resetResult()
{
	kdebugf();
	Tcl_ResetResult(tcl_int);
}

void TclSession::unlockErrors()
{
	kdebugf();
	allowErrors = TRUE;
}

void TclSession::handlePopup(int id)
{
	kdebugf();
	kdebug("Handle popup %d, command code: %s\n", id, tclPopupCmds[id].latin1());
	QCString utf = tclPopupCmds[id].utf8();
	Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
	int ret = Tcl_Eval(tcl_int, Tcl_GetString(result));
	if (ret)
	{
		error(QString::fromUtf8(Tcl_GetStringResult(tcl_int)), TRUE);
	}
}

void TclSession::handleUserMenu(int id)
{
	kdebugf();
	kdebug("Handle user menu %d, command code: %s\n", id, tclMenuCmds[id].latin1());
	int ret = Tcl_Eval(tcl_int, tclMenuCmds[id]);
	if (ret)
	{
		error(QString::fromUtf8(Tcl_GetStringResult(tcl_int)), TRUE);
	}
}

QString TclSession::scriptName()
{
	kdebugf();
	return scriptFile;
}

void TclSession::killAllTimers()
{
	kdebugf();
	QValueList<timerID>::Iterator it;
	for ( it = tclTimers.begin(); it != tclTimers.end(); it++ )
	{
		(*it).ptr->stop();
		delete (*it).ptr;
	}
	tclTimers.clear();
}

void TclSession::setID(int id)
{
	kdebugf();
	sessID = id;
}

int TclSession::getID()
{
	kdebugf();
	return sessID;
}

Tcl_Interp* TclSession::tclInt()
{
	// kdebugf() was generating too much craps here
	return tcl_int;
}

TclEventHandler* TclSession::tclEvtHnd()
{
	kdebugf();
	return tclEventHandler;
}

TclConsole* TclSession::console()
{
	kdebugf();
	return Tcl_cons;
}

QPopupMenu* TclSession::menu()
{
	kdebugf();
	return tclMenu;
}

void TclSession::processOneEvent()
{
	for ( uint i = 0; i < 5; i++ )
		Tcl_DoOneEvent(TCL_ALL_EVENTS|TCL_DONT_WAIT);
}

int TclSession::eval(TclString* str)
{
	return Tcl_Eval(tcl_int, str->dString());
}

int TclSession::eval(QString str)
{
	QCString utf = str.utf8();
	Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
	return Tcl_Eval(tcl_int, Tcl_GetString(result));
}

void TclSession::addTclCommands()
{
	Tcl_CreateCommand(tclInt(), "chat", Tcl_CmdChat, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "configfile", Tcl_CmdConfigFile, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "hint", Tcl_CmdHint, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "ignore", Tcl_CmdIgnore, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "on", Tcl_CmdEvent, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "msg", Tcl_CmdMsg, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "play", Tcl_CmdPlay, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "quit", Tcl_CmdQuit, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "timer", Tcl_CmdTimer, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "user", Tcl_CmdUser, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "status", Tcl_CmdStatus, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "server", Tcl_CmdServer, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "console", Tcl_CmdConsole, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "config", Tcl_CmdConfig, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "usermenu", Tcl_CmdUserMenu, idPtr, NULL);
//	Tcl_CreateCommand(tclInt(), "trayicon", Tcl_CmdTrayIcon, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "parse", Tcl_CmdParse, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "descriptions", Tcl_CmdDescriptions, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "search", Tcl_CmdSearch, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "hist", Tcl_CmdHistory, idPtr, NULL);

	// XMMS module
//	Tcl_CreateCommand(tclInt(), "xmms", Tcl_CmdXmms, idPtr, NULL);

	Tcl_CreateCommand(tclInt(), "popup", Tcl_CmdPopup, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "inputdialog", Tcl_CmdInputDialog, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "filedialog", Tcl_CmdFileDialog, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "msgbox", Tcl_CmdMessageBox, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "qlistbox", Tcl_CmdListBox, idPtr, NULL);
	Tcl_CreateCommand(tclInt(), "qmenu", Tcl_CmdMenu, idPtr, NULL);
}












