#ifndef TCL_SCRIPTING_H
#define TCL_SCRIPTING_H

#include <tcl.h>

#include "kadu.h"
#include "debug.h"
#include "chat.h"

class TclSession;
class TclString;
class TclEventHandler;
class TclTimer;
class TclLineEdit;
class TclTextEdit;
class TclConsole;
class TclListBox;
class TclLoadDialog;
class TclString;
class TclMisc;

struct timerID
{
	int id;
	TclTimer *ptr;
};

struct TclPopup
{
	QPopupMenu *menu;
	int id;
	QMap<int,QString> cmd;
};

typedef QStringList TclEvent;

class TclSession : public QObject
{
	Q_OBJECT

	private:
		typedef QMap<QString,TclPopup> TclPopups;
		typedef QMap<int,TclEvent> TclEvents;
		typedef QValueList<timerID> TclTimers;
		typedef QMap<int,QString> TclPopupCmds;
		typedef QMap<int,QString> TclMenuCmds;
		typedef QMap<QString,int> TclMenuBar;

		bool start();
		void stop();
		void addTclCommands();
		int load();

		TclConsole* Tcl_cons;
		Tcl_Interp* tcl_int;
		TclEventHandler* tclEventHandler;
		QPopupMenu* tclMenu;
		int sessID;
		int* idPtr;
		int menu_id;
		bool isMain;
		bool tclrun;
		QString scriptFile;
		QTimer *hiddingLoop;
		bool allowErrors;

	public:
		TclSession::TclSession(bool main = FALSE);
		TclSession::~TclSession();
		void addMenu();
		void delMenu();
		void menuItemSetEnabled(int idx, bool enabled);
		int load(QString file);
		bool isrun();
		void error(QString cmd, bool line = FALSE);
		void errorFromResult();
		void resetResult();
		QString scriptName();
		void killAllTimers();
		void setID(int);
		int getID();
		Tcl_Interp* tclInt();
		TclEventHandler* tclEvtHnd();
		TclConsole* console();
		QPopupMenu* menu();
		int eval(TclString*);
		int eval(QString);
		
		TclPopups tclPopups;
		TclEvents tclEventList;
		TclTimers tclTimers;
		TclPopupCmds tclPopupCmds;
		TclMenuCmds tclMenuCmds;
		TclMenuBar tclMenuBar;

	public slots:
		void openLoadDialog();
		void openConsole();
		void handlePopup(int id);
		void handleUserMenu(int id);
		void hideTclMenu();
		void showTclMenu();
		void aboutModule();
		void processOneEvent();
		void unlockErrors();
};

typedef QMap<int, TclSession*> TclSessions;
typedef QMap<int, SearchRecord*> PubDirId;

extern TclSessions tclSess;
extern TclLoadDialog *loadDialog;
extern QTimer* events_loop;
extern PubDirId tclPubDirList;
extern TclMisc *tclMisc;

#endif
