/***************************************************************************
 *																		 *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	 *
 *   (at your option) any later version.								   *
 *																		 *
 ***************************************************************************/

#include <qcombobox.h>

#include "tcl_scripting.h"
#include "tclconsole.h"
#include "tcllineedit.h"
#include "tcltextedit.h"

Tcl_ChannelType tclConsoleOut;

TclConsole::TclConsole()
{
	kdebugf();
	setCaption(tr("TCL Console"));
	setMinimumSize(400, 200);
	resize(600, 400);

	// Results
	QFont *font = new QFont("misc-fixed");
	font->setPointSize(12);
	font->setBold(TRUE);

	txt = new TclTextEdit(this);
	txt->setReadOnly(TRUE);
	txt->setCurrentFont(*font);
	txt->setColor(QColor("gray"));
	txt->setPaper(QColor("black"));
	txt->setMinimumSize(300, 130);
	
	// Command line
	cmd = new TclLineEdit(this);
	cmd->setFont(*font);
	cmd->setPaletteForegroundColor(QColor("white"));
	cmd->setPaletteBackgroundColor(QColor("black"));
	cmd->setMinimumWidth(240);

	// Executing button
	QPushButton *btn = new QPushButton(this);
	btn->setText("Execute");
	
	// TCL context choosing box
	cbox = new QComboBox(FALSE, this);

	// Command line and button
	QGridLayout *g = new QGridLayout(this, 1, 1);
	g->addMultiCellWidget(txt, 0, 12, 0, 20);
	g->addMultiCellWidget(cmd, 13, 13, 0, 14);
	g->addMultiCellWidget(btn, 13, 13, 15, 16);
	g->addMultiCellWidget(cbox, 13, 13, 17, 20);

	// Command line and results
	connect(btn, SIGNAL(clicked()), this, SLOT(exec()));
	connect(cmd, SIGNAL(enter()), this, SLOT(exec()));
}

void TclConsole::mousePressEvent(QMouseEvent* e)
{
	kdebugf();
	getCmdLine()->setFocus();
}

TclLineEdit* TclConsole::getCmdLine()
{
	kdebugf();
	return cmd;
}

void TclConsole::exec()
{
	kdebugf();
	QString code = cmd->text(), file;
	cmd->clear();
	uint i = 0;
	while ( i < tclSess.count() && QStringList::split("/", tclSess[i]->scriptName()).last() != cbox->currentText() )
		i++;

	if (i == tclSess.count())
		i = 0;

	Tcl_Obj* result;
	QCString utf = code.utf8();
	result = Tcl_NewStringObj((const char* const)utf, utf.length());
	Tcl_Eval(tclSess[i]->tclInt(), Tcl_GetString(result));
	puts(QString::fromUtf8(Tcl_GetStringResult(tclSess[i]->tclInt())));
	cmd->histCnt = 0;
	cmd->history[0] = code;
	cmd->history.prepend("");
	uint max = cmd->consoleHistory + 1;
	while (cmd->history.count() > max)
	{
		QStringList::iterator it = cmd->history.end();
		it--;
		cmd->history.remove(it);
	}
}

void TclConsole::puts(QString str)
{
	kdebugf();
	txt->setReadOnly(FALSE);
	txt->append(str);
	txt->setReadOnly(TRUE);
}

QComboBox* TclConsole::context()
{
	kdebugf();
	return cbox;
}
