/***************************************************************************
 *																		 *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	 *
 *   (at your option) any later version.								   *
 *																		 *
 ***************************************************************************/

#include "tcllineedit.h"
#include "debug.h"
 
TclLineEdit::TclLineEdit(QWidget *parent, const char *name) : QLineEdit(parent, name)
{
	kdebugf();
	consoleHistory = 10;
}

void TclLineEdit::keyPressEvent(QKeyEvent *e)
{
	/*
		kdebugf() is ommited, becouse of craps it was generating.
	*/
	QString str;

	if (e->state() == Qt::ControlButton)
	{
		switch (e->key())
		{
			case Qt::Key_C:
				copy();
				break;
			case Qt::Key_X:
				cut();
				break;
			case Qt::Key_V:
				paste();
				break;
			default:
				e->ignore();
		}
	}
	else if (e->state() == Qt::ShiftButton)
	{
		switch (e->key())
		{
			case Qt::Key_Left:
				cursorBackward(TRUE);
				break;
			case Qt::Key_Right:
				cursorForward(TRUE);
				break;
			default:
				insert(e->text());
		}
	}
	else if (e->state() == Qt::KeyButtonMask)
	{
		e->ignore();
	}
	else
	{
		switch (e->key())
		{
			case Qt::Key_Up:
				historyUp();
				break;
			case Qt::Key_Down:
				historyDown();
				break;
			case Qt::Key_PageUp:
				e->ignore();
				break;
			case Qt::Key_PageDown:
				e->ignore();
				break;
			case Qt::Key_Left:
				cursorBackward(FALSE);
				break;
			case Qt::Key_Right:
				cursorForward(FALSE);
				break;
			case Qt::Key_Backspace:
				if (e->state() == Qt::ShiftButton)
				{
					cursorWordBackward(TRUE);
				}
				backspace();
				break;
			case Qt::Key_Delete:
				if (e->state() == Qt::ShiftButton)
				{
					cursorWordForward(TRUE);
				}
				del();
				break;
			case Qt::Key_Home:
				home(FALSE);
				break;
			case Qt::Key_End:
				end(FALSE);
				break;
			case Qt::Key_Return:
			case Qt::Key_Enter:
				emit enter();
				break;
			default:
				insert(e->text());
		}
	}
}

void TclLineEdit::historyUp()
{
	kdebugf();
	if (histCnt < consoleHistory && histCnt < history.count())
	{
		histCnt++;
		setText(history[histCnt]);
	}
}

void TclLineEdit::historyDown()
{
	kdebugf();
	if (histCnt > 0)
	{
		histCnt--;
		setText(history[histCnt]);
	}
}
