/***************************************************************************
 *																		 *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	 *
 *   (at your option) any later version.								   *
 *																		 *
 ***************************************************************************/

// Kadu includes
#include "debug.h"

// QT includes
#include <qlistbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include "tcllistbox.h"

TclListBox::TclListBox(bool mesg)
{
	kdebugf();
	setMinimumWidth(400);
	setMinimumHeight(300);

	QPushButton *btn_ok = new QPushButton(this);
	btn_ok->setText(tr("OK"));
	btn_ok->setDefault(FALSE);

	QPushButton *btn_ccl = new QPushButton(this);
	btn_ccl->setText(tr("Cancel"));

	QPushButton *btn_add = new QPushButton(this);
	btn_add->setText(tr("Add"));
	btn_add->setDefault(TRUE);

	QPushButton *btn_del = new QPushButton(this);
	btn_del->setText(tr("Delete"));

	toAdd = new QLineEdit(this);
	toAdd->setMinimumWidth(100);
	toAdd->setFocus();

	_list = new QListBox(this);
	_list->setMinimumWidth(400);
	_list->setMinimumHeight(200);

	QGridLayout *g = new QGridLayout(this, 1, 1);
	if (mesg)
	{
		QVBoxLayout *l = new QVBoxLayout(this);
		msg = new QLabel(this);
		l->addWidget(msg);
		l->setMargin(10);
		g->addMultiCellLayout(l, 0, 0, 0, 20);
		g->addMultiCellWidget(toAdd, 1, 1, 0, 10);
		g->addMultiCellWidget(btn_add, 1, 1, 11, 14);
		g->addMultiCellWidget(btn_del, 1, 1, 16, 19);
		g->addMultiCellWidget(_list, 2, 22, 0, 20);
		g->addMultiCellWidget(btn_ok, 24, 24, 1, 5);
		g->addMultiCellWidget(btn_ccl, 24, 24, 14, 19);
	}
	else
	{
		g->addMultiCellWidget(toAdd, 0, 0, 0, 10);
		g->addMultiCellWidget(btn_add, 0, 0, 11, 14);
		g->addMultiCellWidget(btn_del, 0, 0, 16, 19);
		g->addMultiCellWidget(_list, 1, 21, 0, 20);
		g->addMultiCellWidget(btn_ok, 23, 23, 1, 5);
		g->addMultiCellWidget(btn_ccl, 23, 23, 14, 19);
	}

	connect(btn_ok, SIGNAL(clicked()), this, SLOT(ok()));
	connect(btn_ccl, SIGNAL(clicked()), this, SLOT(cancel()));
	connect(btn_add, SIGNAL(clicked()), this, SLOT(add()));
	connect(btn_del, SIGNAL(clicked()), this, SLOT(del()));
}

void TclListBox::add()
{
	kdebugf();
	_list->insertItem(toAdd->text());
	toAdd->clear();
}

void TclListBox::del()
{
	kdebugf();
	_list->removeItem(_list->currentItem());
}

void TclListBox::ok()
{
	kdebugf();
	accept();
}

void TclListBox::cancel()
{
	kdebugf();
	reject();
}

void TclListBox::setMsg(QString mesg)
{
	kdebugf();
	msg->setText(mesg);
}

QListBox* TclListBox::list()
{
	kdebugf();
	return _list;
}

void TclListBox::keyPressEvent(QKeyEvent* e)
{
	kdebugf();
	switch (e->key())
	{
		case Qt::Key_Delete:
			del();
			break;
		case Qt::Key_Return:
		case Qt::Key_Enter:
			add();
			break;
	}
}

