/***************************************************************************
 *																		 *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or	 *
 *   (at your option) any later version.								   *
 *																		 *
 ***************************************************************************/

#include "tclmisc.h"

QString TclMisc::GGStatusToTclStatus(int status)
{
	switch (GG_S(status))
	{
		case GG_STATUS_NOT_AVAIL:
			return "notavailable";
			break;
		case GG_STATUS_AVAIL:
			return "available";
			break;
		case GG_STATUS_BUSY:
			return "busy";
			break;
		case GG_STATUS_AVAIL_DESCR:
			return "available+description";
			break;
		case GG_STATUS_BUSY_DESCR:
			return "busy+description";
			break;
		case GG_STATUS_NOT_AVAIL_DESCR:
			return "notavailable+description";
			break;
		case GG_STATUS_INVISIBLE:
			return "invisible";
			break;
		case GG_STATUS_INVISIBLE_DESCR:
			return "invisible+description";
			break;
		case GG_STATUS_BLOCKED:
			return "blocked";
			break;
		default:
			return "";
	}
}

int TclMisc::TclStatusToGGStatus(QString status)
{
	QStringList stats = QStringList::split(" ", "notavailable available busy available+description busy+description notavailable+description invisible invisible+description");
	int id = stats.findIndex(status);
	switch (id)
	{
		case 0:
			return GG_STATUS_NOT_AVAIL;
			break;
		case 1:
			return GG_STATUS_AVAIL;
			break;
		case 2:
			return GG_STATUS_BUSY;
			break;
		case 3:
			return GG_STATUS_AVAIL_DESCR;
			break;
		case 4:
			return GG_STATUS_BUSY_DESCR;
			break;
		case 5:
			return GG_STATUS_NOT_AVAIL_DESCR;
			break;
		case 6:
			return GG_STATUS_INVISIBLE;
			break;
		case 7:
			return GG_STATUS_INVISIBLE_DESCR;
			break;
		default:
			return -1;
	}
}

void TclMisc::getKey(QKeyEvent* e, QString& keystr, QString& modstr)
{
	switch (e->state())
	{
		case Qt::ShiftButton:
			modstr = "shift";
			break;
		case Qt::ControlButton:
			modstr = "control";
			break;
		case Qt::AltButton:
			modstr = "alt";
			break;
		case Qt::MetaButton:
			modstr = "meta";
			break;
		case Qt::Keypad:
			modstr = "keypad";
			break;
		default:
			break;
	}

	int key = e->key();
	if (keystr[0].isPrint())
		return;

	switch (key)
	{
		case Qt::Key_Super_L:
			keystr = "win_l";
			break;
		case Qt::Key_Super_R:
			keystr = "win_r";
			break;
		case Qt::Key_Shift:
			keystr = "shift";
			break;
		case Qt::Key_Control:
			keystr = "control";
			break;
		case Qt::Key_Meta:
			keystr = "meta";
			break;
		case Qt::Key_Alt:
			keystr = "alt";
			break;
		case Qt::Key_Escape:
			keystr = "escape";
			break;
		case Qt::Key_Tab:
			keystr = "tab";
			break;
		case Qt::Key_Backspace:
			keystr = "backspace";
			break;
		case Qt::Key_Return:
		case Qt::Key_Enter:
			keystr = "enter";
			break;
		case Qt::Key_Insert:
			keystr = "insert";
			break;
		case Qt::Key_Delete:
			keystr = "delete";
			break;
		case Qt::Key_Pause:
			keystr = "pause";
			break;
		case Qt::Key_Print:
			keystr = "print";
			break;
		case Qt::Key_SysReq:
			keystr = "sysreq";
			break;
		case Qt::Key_Home:
			keystr = "home";
			break;
		case Qt::Key_End:
			keystr = "end";
			break;
		case Qt::Key_Left:
			keystr = "left";
			break;
		case Qt::Key_Up:
			keystr = "up";
			break;
		case Qt::Key_Right:
			keystr = "right";
			break;
		case Qt::Key_Down:
			keystr = "down";
			break;
		case Qt::Key_Prior:
			keystr = "pgup";
			break;
		case Qt::Key_Next:
			keystr = "pgdown";
			break;
		case Qt::Key_CapsLock:
			keystr = "capslock";
			break;
		case Qt::Key_NumLock:
			keystr = "numlock";
			break;
		case Qt::Key_ScrollLock:
			keystr = "scrolllock";
			break;
		case Qt::Key_Clear:
			keystr = "clear";
			break;
		case Qt::Key_F1:
			keystr = "f1";
			break;
		case Qt::Key_F2:
			keystr = "f2";
			break;
		case Qt::Key_F3:
			keystr = "f3";
			break;
		case Qt::Key_F4:
			keystr = "f4";
			break;
		case Qt::Key_F5:
			keystr = "f5";
			break;
		case Qt::Key_F6:
			keystr = "f6";
			break;
		case Qt::Key_F7:
			keystr = "f7";
			break;
		case Qt::Key_F8:
			keystr = "f8";
			break;
		case Qt::Key_F9:
			keystr = "f9";
			break;
		case Qt::Key_F10:
			keystr = "f10";
			break;
		case Qt::Key_F11:
			keystr = "f11";
			break;
		case Qt::Key_F12:
			keystr = "f12";
			break;
		case Qt::Key_Menu:
			keystr = "menu";
			break;
		default:
			keystr = "unknown";
	}
}

QString TclMisc::GaduErrorToString(GaduError err)
{
	switch (err)
	{
		case ConnectionServerNotFound:
			return "Server not found";
			break;
		case ConnectionCannotConnect:
			return "Cannot connect";
			break;
		case ConnectionNeedEmail:
			return "Need email";
			break;
		case ConnectionInvalidData:
			return "Invalid data";
			break;
		case ConnectionCannotRead:
			return "Cannot read";
			break;
		case ConnectionCannotWrite:
			return "Cannot write";
			break;
		case ConnectionIncorrectPassword:
			return "Incorrect password";
			break;
		case ConnectionTlsError:
			return "TSL error";
			break;
		case ConnectionUnknow:
			return "Unknown";
			break;
		case ConnectionTimeout:
			return "Timeout";
			break;
		case Disconnected:
			return "Disconnected";
			break;
		default:
			return "";
	}
}

TclMisc::TclMisc()
{
	tclEventDef["ack"] = 10;
	tclEventDef["chat0"] = 20;
	tclEventDef["chat1"] = 30;
	tclEventDef["chat2"] = 40;
	tclEventDef["chatkey"] = 50;
	tclEventDef["chatkeyrel"] = 52;
	tclEventDef["chmystatus"] = 60;
	tclEventDef["chstatus"] = 70;
	tclEventDef["closedchat"] = 80;
	tclEventDef["closingchat"] = 90;
	tclEventDef["configopen"] = 94;
	tclEventDef["configapply"] = 95;
	tclEventDef["configclose"] = 96;
	tclEventDef["connected"] = 100;
	tclEventDef["connecting"] = 110;
	tclEventDef["connerr"] = 120;
	tclEventDef["disconnected"] = 130;
	tclEventDef["key"] = 136;
	tclEventDef["listreply"] = 140;
	tclEventDef["sysmsg"] = 150;
	tclEventDef["msg_blocked"] = 152;
	tclEventDef["msg_delivered"] = 153;
	tclEventDef["msg_queued"] = 154;
	tclEventDef["msg_boxfull"] = 155;
	tclEventDef["msg_notdelivered"] = 156;
	tclEventDef["msg_accepted"] = 157;
	tclEventDef["msg_rejected"] = 158;
	tclEventDef["msgsent"] = 160;
	tclEventDef["newchat"] = 170;
	tclEventDef["progresscancel"] = 180;
	tclEventDef["results"] = 190;
	tclEventDef["searchdone"] = 200;
	tclEventDef["searchnext"] = 210;
	tclEventDef["sendmsg1"] = 220;
	tclEventDef["sendmsg2"] = 230;
	tclEventDef["unloadmodule"] = 240;
	tclEventDef["unloadscript"] = 245;
	tclEventDef["usl"] = 250;
	tclEventDef["usldns"] = 260;
	tclEventDef["uslexp"] = 270;
	tclEventDef["uslclr"] = 280;
	tclEventDef["uslimp"] = 290;
}

int TclMisc::getEventId(QString name)
{
	return tclEventDef[name];
}

QString TclMisc::getEventString(int id)
{
	QMap<QString,int>::Iterator it;
	for( it = tclEventDef.begin(); it != tclEventDef.end(); it++ )
		if (it.data() == id)
			return it.key();
	return "";
}

QStringList TclMisc::getEvents()
{
	return tclEventDef.keys();
}

void TclMisc::setTclResult(Tcl_Interp* interp, QString str)
{
	QCString utf = str.utf8();
	Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
	Tcl_SetResult(interp, Tcl_GetString(result), TCL_VOLATILE);
}

void TclMisc::appendTclResult(Tcl_Interp* interp, QString str)
{
	QCString utf = str.utf8();
	Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
	Tcl_AppendElement(interp, Tcl_GetString(result));
}

bool TclMisc::isDescStatus(int status)
{
	switch (GG_S(status))
	{
		case GG_STATUS_NOT_AVAIL:
		case GG_STATUS_INVISIBLE:
		case GG_STATUS_AVAIL:
		case GG_STATUS_BUSY:
		case GG_STATUS_BLOCKED:
			return FALSE;
			break;
		case GG_STATUS_AVAIL_DESCR:
		case GG_STATUS_BUSY_DESCR:
		case GG_STATUS_NOT_AVAIL_DESCR:
		case GG_STATUS_INVISIBLE_DESCR:
			return TRUE;
			break;
		default:
			return FALSE;
	}
}

void TclMisc::setTclStatus(QString stat, QString desc)
{
	QStringList stats = QStringList::split(" ", "notavailable available busy available+description busy+description notavailable+description invisible invisible+description");
	int id = stats.findIndex(stat);
	switch (id)
	{
		case 0:
		case 5:
			gadu->status().setOffline(desc);
			break;
		case 1:
		case 3:
			gadu->status().setOnline(desc);
			break;
		case 2:
		case 4:
			gadu->status().setBusy(desc);
			break;
		case 6:
		case 7:
			gadu->status().setInvisible(desc);
			break;
	}
}

QString TclMisc::KaduStatusToTclStatus(int index)
{
	switch (index)
	{
		case 0:
			return "available";
			break;
		case 1:
			return "available+description";
			break;
		case 2:
			return "busy";
			break;
		case 3:
			return "busy+description";
			break;
		case 4:
			return "invisible";
			break;
		case 5:
			return "invisible+description";
			break;
		case 6:
			return "notavailable";
			break;
		case 7:
			return "notavailable+description";
			break;
		case 8:
			return "blocked";
			break;
		default:
			return "";
	}
}

int TclMisc::TclStatusToKaduStatus(QString status)
{
	QStringList stats = QStringList::split(" ", "available available+description busy busy+description invisible invisible+description notavailable notavailable+description");
	return stats.findIndex(status);
}

QTextEdit::CursorAction TclMisc::tclIndexToQtIndex(QString idxstr)
{
	QStringList idxs = QStringList::split(" ", "end linestart lineend lineup linedown nextword prevword");
	int idx = idxs.findIndex(idxstr);
	switch (idx)
	{
		case 0:
			return QTextEdit::MoveEnd;
			break;
		case 1:
			return QTextEdit::MoveLineStart;
			break;
		case 2:
			return QTextEdit::MoveLineEnd;
			break;
		case 3:
			return QTextEdit::MoveUp;
			break;
		case 4:
			return QTextEdit::MoveDown;
			break;
		case 5:
			return QTextEdit::MoveWordForward;
			break;
		case 6:
			return QTextEdit::MoveWordBackward;
			break;
		default:
		{
			return QTextEdit::MoveHome;
		}
	}
}

QStringList TclMisc::tclListToQtList(QString str)
{
	Tcl_DString ds;
	Tcl_DStringInit(&ds);
	QCString utf = str.utf8();
	Tcl_Obj* result = Tcl_NewStringObj((const char* const)utf, utf.length());
	return tclListToQtList( Tcl_GetString(result) );
}

QStringList TclMisc::tclListToQtList(const char* str)
{
	QStringList list;
	const char** argv;
	int argc;
	if (Tcl_SplitList(NULL, str, &argc, &argv) == TCL_OK)
	{
		for ( int i = 0; i < argc; i++ )
			list.append(QString::fromUtf8(argv[i]));
	}
	return list;
}
